/**
 *
 */

class GoogleAddressValidation {
    constructor(ApiKey) {
        this.ApiKey = ApiKey;
    }

    getGoogleValidation(addressObject) {

        return new Promise((resolve, reject) => {
            let url = `https://addressvalidation.googleapis.com/v1:validateAddress?key=${this.ApiKey}`;
            console.log(addressObject);

            //make a post request to google address validation api
            fetch(url, {
                method: 'POST',
                body: JSON.stringify(addressObject)
            }).then((response) => {
                return response.json();
            }).then((data) => {
                resolve(data);
            }).catch((error) => {
                resolve({error: error});
            });
        });
    }

    validateAddress(addressObject) {
        let CustomError = {
            AddressComponentsAreValid: false,
            inputDetailIsValid: false,
            result: false,
            incomplete: true,
            data: null
        };

        if (!Array.isArray(addressObject.address.addressLines)) {
            console.log(addressObject, 'invalid');
            return this.promise( CustomError)
        }

        if(addressObject.address.addressLines.length === 0 || addressObject.address.addressLines[0] === "") {
            console.log('invalid address');
            return this.promise( CustomError);
        }

        return new Promise((resolve, reject) => {
           this.getGoogleValidation(addressObject).then((data) => {
                if(data.error) {
                     reject(data.error);
                } else {
                    let AddressComponentsAreValid = this.checkAddressComponents(data.result);
                    let inputDetailIsValid = this.checkInputDetail(data.result);

                    resolve({
                        AddressComponentsAreValid: AddressComponentsAreValid,
                        inputDetailIsValid: inputDetailIsValid,
                        result: AddressComponentsAreValid && inputDetailIsValid,
                        data: data.result
                    });
                }
           }).catch((error) => {
               reject(error);
           });
        });
    }

    /**
     * Check if the address is valid
     * @param data
     * @returns {boolean}
     */
    checkAddressComponents(data) {
        let valid = false;
        //count how many addressComponents are in the address
        let addressComponents = Object.keys(data.address.addressComponents).length;

        //count the addressComponents with confirmationLevel of "CONFIRMED"
        let confirmedAddressComponents = 0;
        for (let key in data.address.addressComponents) {
            if (data.address.addressComponents[key].confirmationLevel === "CONFIRMED") {
                confirmedAddressComponents++;
            }
        }

        //compare the number of addressComponents with the number of confirmed addressComponents
        //if addressComponents.length - 1 <= confirmedAddressComponents.length, then the address is valid
        if (addressComponents - 1 <= confirmedAddressComponents) {
            valid = true;
        }

        return valid;
    }

    getFriendlyAddressParts(address){

        if (!address.hasOwnProperty('missingComponentTypes') && !address.hasOwnProperty('unconfirmedComponentTypes')) {
            return [];
        }

        let replacements = {
            'STREET_NUMBER': 'Street Number',
            'ROUTE': 'Street Name',
            'LOCALITY': 'Suburb',
            'ADMINISTRATIVE_AREA_LEVEL_1': 'State',
            'POSTAL_CODE': 'Postcode',
            'COUNTRY': 'Country',
        };


        let friendlyAddressParts = [];

        if(address.missingComponentTypes) {
            address.missingComponentTypes.forEach((part) => {
                part = part.toUpperCase();
                console.log(part, Object.keys(replacements), Object.keys(replacements).hasOwnProperty(part));

                if (replacements.hasOwnProperty(part) && !friendlyAddressParts.includes(replacements[part])) {
                    friendlyAddressParts.push(replacements[part]);

                    //remove from missingComponentTypes
                    delete address.missingComponentTypes[ address.missingComponentTypes.indexOf(part) ];
                }
            });

        }

        if(address.unconfirmedComponentTypes){
            address.unconfirmedComponentTypes.forEach((part) => {
                part = part.toUpperCase();

                if (replacements.hasOwnProperty(part) && !friendlyAddressParts.includes(replacements[part])) {
                    friendlyAddressParts.push(replacements[part]);

                    //remove from unconfirmedComponentTypes
                    delete address.unconfirmedComponentTypes[ address.unconfirmedComponentTypes.indexOf(part) ];
                }
            });
        }

        //remove duplicates
        friendlyAddressParts = [...new Set(friendlyAddressParts)];

        return friendlyAddressParts;
    }

    /**
     * Check if the input granularity and validation granularity are both "PREMISE"
     * @param data
     * @return {boolean}
     */
    checkInputDetail(data) {
        let valid = false;

        if(data.verdict.inputGranularity === "PREMISE" &&
            ([
                'OTHER',
                'PREMISE',
                'SUB_PREMISE',
                'PREMISE_PROXIMITY'
            ].indexOf(data.verdict.validationGranularity) > -1)) {
            valid = true;
        }

        return valid;
    }

    promise(resolveObject) {
        return new Promise((resolve, reject) => {
            resolve(resolveObject );
        });
    }
}

export default GoogleAddressValidation;