<?php

namespace JimsCustomerBooking\Helpers;

/**
 * This class offers helper methods for time
 */
class TimeHelper extends Helper
{
    /**
     * @param string $date
     * @param string $timezone , default is UTC, other options are 'Europe/London', Melbourne/Australia
     * @param string $format_from
     * @param string $format_to
     * @param string $returnType
     * @return string|\DateTime
     */
    public static function convertToUTC($date, $timezone = 'UTC', $format_from = 'd/m/Y H:i:s', $format_to = 'd/m/Y', $returnType = 'string')
    {

        if (empty($date)) return $date;

        if (is_numeric($date)) {
            $date = date($format_from, $date);
        }

        if (!self::isValidTimeZone($timezone)) {
            $timezone = 'UTC';
        }

        //set time to 00:01 and remove spaces
        $time = '00:00:01';

        $date = implode(' ', [$date, $time]);

        try {
            $date = \DateTime::createFromFormat($format_from, $date, new \DateTimeZone($timezone));
            $date->setTimezone(new \DateTimeZone('UTC'));
            return $returnType == 'string' ? $date->format($format_to) : $date;
        } catch (\Exception $e) {
            error_log($e->getMessage());
        }

        return $date;
    }

    /**
     * @param string $timezone
     * @return string
     */
    public static function getTimezoneOffset($timezone = 'UTC')
    {
        $date = new \DateTime('now', new \DateTimeZone($timezone));
        return $date->format('P');
    }

    /**
     * @param $countryCode
     * @return string
     */
    public static function getTimezoneFromCountryCode($countryCode)
    {
        $timezone = '';
        switch ($countryCode) {
            case 'AU':
                $timezone = 'Australia/Melbourne';
                break;
            case 'NZ':
                $timezone = 'Pacific/Auckland';
                break;
            case 'CA':
                $timezone = 'America/Toronto';
                break;
            case 'GB':
                $timezone = 'Europe/London';
                break;
            case 'US':
                $timezone = 'America/New_York';
                break;
            default:
                $timezone = 'UTC';
                break;
        }
        return $timezone;
    }

    /**
     * @param $date
     * @param $countryCode
     * @param string $format
     * @return string
     */
    public static function convertDateToTimezoneFromCountryCode($date, $countryCode)
    {
        $timezone = self::getTimezoneFromCountryCode($countryCode);
        return self::convertToUTC($date, $timezone);
    }

    public static function isValidTimeZone($timezone)
    {
        try {
            new \DateTimeZone($timezone);
            return true;
        } catch (\Exception $e) {
            return false;
        }
    }
}